'' =================================================================================================
''
''   File....... jm_ble_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2017-2020 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 25 AUG 2020
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  BR_TERM  = 115_200                                            ' terminal baud rate
  BR_BLE   =   9_600                                            ' ble baud rate

  OUT_MS   = 250                                                ' pulse outputs for 1/4s
  

con { fixed io pins }                                                  
                                                                 
  RX1   = 31  { I }                                             ' serial / programming
  TX1   = 30  { O }                                            
                                                              
  SDA1  = 29  { I/O }                                           ' i2c / eeprom
  SCL1  = 28  { I/O } 


con { app io pins }
  
  LOWER =  9  { O }
  RAISE =  8  { O } 

  RX2   =  5  { I }                                             ' HM-10 serial
  TX2   =  4  { O }


con

  #0, C_STOP, C_RAISE, C_LOWER, C_TIMER

  CMD_LEN = 5                                                   ' length of shortest command string

   
con

  #true, ON, OFF                                                ' digital control

  BUF_SIZE = 81


obj

' main                                                          ' * master Spin cog
  time       : "jm_time_80"                                     '   timing and delays
  io         : "jm_io_basic"                                    '   essential io
  tbuf       : "jm_textbuf"                                     '   text buffer management
  ble        : "jm_fullduplexserial"                            ' * serial IO for BLE module    
  term       : "jm_fullduplexserial"                            ' * serial IO for terminal
' background                                                    ' * background timer/outs control   
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  byte  buffer[BUF_SIZE]

  byte  command
                                                                 
                                                                 
pub main                                                      
                                                                 
  setup

  term.rx
  term.tx(term#CLS)
  term.str(string("HM-10 Remote Demo", 13, 13))

  term.rxflush
  ble.rxflush
  
  repeat
    if (get_command)
      process_command


pub get_command : c

  tbuf.clear                                                    ' clear input

  repeat
    c := ble.rxtime(5)                                          ' check for byte from BLE
    if ((c > 31) and (c < 128))                                 ' if ascii   
      tbuf.putc(tbuf.upper(c))                                  '  add to buffer
    else
      return tbuf.size                                          ' return on time-out/non-ascii
        

dat

  CmdRaise      byte    "RAISE", 0
  CmdLower      byte    "LOWER", 0
  CmdTimer      byte    "TIMER", 0


pub process_command | idx, c

  if (strcomp(@CmdRaise, tbuf.str))
    movetimer := -OUT_MS                                        ' activate timer
    command := C_RAISE                                          ' control io
    term.str(string(13, "Raise pulse", 13))                     ' report to terminal
    return

  if (strcomp(@CmdLower, tbuf.str))
    movetimer := -OUT_MS    
    command := C_LOWER     
    term.str(string(13, "Lower pulse", 13))   
    return

  if (strcomp(@CmdTimer, tbuf.str))    
    term.str(string(13, "Millis = "))
    term.dec(millis)
    term.tx(13)   
    return

  movetimer := 0
  command := C_STOP
  
  term.str(string(13, "Unrecognized command: ", 13, "-- "))
  repeat idx from 0 to tbuf.size-1
    c := buffer[idx]
    term.fhex(c, 2)
    term.tx(" ")
  term.str(string(13, "-- "))
  repeat idx from 0 to tbuf.size-1
    c := buffer[idx]
    if (c => 32)
      term.tx(c)
    else
      term.tx(".")
  term.tx(13)    

                                                                                
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  tbuf.use(@buffer, BUF_SIZE-1)
  tbuf.clear

  ble.start(RX2, TX2, %0000, BR_BLE)                            ' start serial for HM-10 *
                                                              
  term.start(RX1, TX1, %0000, BR_TERM)                          ' start serial for terminal *

  bcog := cognew(background, @bstack) + 1                       ' start background cog *
  

con { background }

var

  long  bcog
  long  bstack[16]

  long  millis                                                  ' general timer
  long  movetimer                                               ' movement timer
  

pri background : t

  io.low(RAISE)
  io.low(LOWER)

  millis := 0
  movetimer := 0

  t := cnt
  repeat
    waitcnt(t += MS_001)
    ++millis

    if (++movetimer => 0)                                       ' timer expired?
      command := C_STOP

    case command
      C_STOP :
        io.low(RAISE)
        io.low(LOWER)

      C_RAISE :
        io.high(RAISE)
        io.low(LOWER)

      C_LOWER :
        io.low(RAISE)
        io.high(LOWER)        
    
                          
con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      